from importlib.metadata import version

from .exceptions import InvalidSceneError, OrbitNotFoundError
from .s1_orbits import API_URL, fetch_for_scene


__version__ = version(__name__)

__all__ = [
    '__version__',
    'fetch_for_scene',
    'API_URL',
    'InvalidSceneError',
    'OrbitNotFoundError',
]



class InvalidSceneError(Exception):
    def __init__(self, scene: str):
        self.scene = scene
        self.message = f'{scene} is not a valid Sentinel-1 scene name.'

    def __str__(self):
        return str(self.message)


class OrbitNotFoundError(Exception):
    def __init__(self, scene: str):
        self.scene = scene
        self.message = f'No orbit file could be found for the provided Sentinel-1 scene: {scene}'

    def __str__(self):
        return str(self.message)



from pathlib import Path
from typing import Union

import requests

from .exceptions import InvalidSceneError, OrbitNotFoundError


API_URL = 'https://s1-orbits.asf.alaska.edu/scene'


def fetch_for_scene(
    scene: str,
    dir: Union[Path, str] = '.',
) -> Path:
    """For the given scene, downloads the AUX_POEORB file if available, otherwise downloads the AUX_RESORB file.

    Args:
        scene: The scene name for which to download the orbit file.
        dir: The directory that the orbit file should download into.

    Raises:
        InvalidSceneError: Thrown if the scene name is not a proper Sentinel-1 scene name.
        OrbitNotFoundError: Thrown if an orbit cannot be found for the provided scene.

    Returns:
        download_path: The path to the downloaded file.
    """
    request_url = f'{API_URL}/{scene}'

    res = requests.get(request_url)
    if res.status_code == 400:
        raise InvalidSceneError(scene)
    if res.status_code == 404:
        raise OrbitNotFoundError(scene)
    res.raise_for_status()

    filename = res.url.split('/')[-1]
    download_path = Path(dir) / filename
    download_path.write_text(res.text)

    return download_path



from pathlib import Path

import pytest
import responses

import s1_orbits


@responses.activate
def test_fetch_for_scene(tmp_path):
    granule = 'foo'
    filename = 'filename.txt'
    file_contents = 'This is some text.'
    request_url = f'{s1_orbits.API_URL}/{granule}'
    response_url = f'https://bar.com/{filename}'

    responses.add(responses.GET, request_url, status=302, headers={'Location': response_url})
    responses.add(
        responses.GET,
        response_url,
        status=200,
        body=file_contents,
        content_type='text/plain',
    )
    filepath = s1_orbits.fetch_for_scene(granule, tmp_path)
    assert filepath == Path(tmp_path) / filename
    assert filepath.read_text() == file_contents

    responses.add(responses.GET, request_url, status=400)
    with pytest.raises(s1_orbits.InvalidSceneError) as invalid_error:
        s1_orbits.fetch_for_scene(granule, tmp_path)
        assert invalid_error.scene == granule

    responses.add(responses.GET, request_url, status=404)
    with pytest.raises(s1_orbits.OrbitNotFoundError) as not_found_error:
        s1_orbits.fetch_for_scene(granule, tmp_path)
        assert not_found_error.scene == granule
